@@ -1,4 +1,5 @@
 from .misc import *
+from .losses import *
 from .models import *
 from .modules import *
 from .trainer import *
@@ -9,11 +10,13 @@ from .data import *
 from .api import cv
 from .api import dl
 from .api import ml
+from .api import nlp
 from .api import multimodal
 from .api import interface as api
 from .api.zoo import *
 from .api.register import *
 from . import dist
+from . import scripts
 
 from pkg_resources import get_distribution
 
