@@ -12,14 +12,15 @@ from typing import Optional
 from skimage.transform import resize
 from torchvision.transforms import transforms
 from torchvision.transforms import InterpolationMode
+from cfcv.misc.toolkit import to_rgb
+from cfcv.misc.toolkit import min_max_normalize
+from cfcv.misc.toolkit import imagenet_normalize
 
 from .A import *
 from .pt import *
 from .general import *
 from .....data import Compose
 from .....data import Transforms
-from .....misc.toolkit import min_max_normalize
-from .....misc.toolkit import imagenet_normalize
 
 
 @Transforms.register("for_generation")
@@ -149,7 +150,7 @@ class SSLTransform(Transforms):
             )
 
         def __call__(self, image: Image) -> Image:
-            image = image.convert("RGB")
+            image = to_rgb(image)
             crops = [self.global_transform1(image), self.global_transform2(image)]
             for _ in range(self.local_crops_number):
                 crops.append(self.local_transform(image))
@@ -188,7 +189,7 @@ class SSLTestTransform(Transforms):
         self.larger_size = int(round(img_size * 8.0 / 7.0))
 
     def fn(self, img: Image.Image) -> np.ndarray:
-        img = img.convert("RGB")
+        img = to_rgb(img)
         img.thumbnail((self.larger_size, self.larger_size), Image.ANTIALIAS)
         img_arr = np.array(img)
         resized_img = resize(img_arr, (self.img_size, self.img_size), mode="constant")
