@@ -5,18 +5,19 @@ from tqdm import tqdm
 from torch import Tensor
 from typing import List
 from typing import Tuple
+from cfcv.misc.toolkit import to_rgb
 
+from .utils import predict_folder
 from ..data import SSLTestTransform
 from ..data import InferenceImageFolderData
 from ..pipeline import SimplePipeline
-from ..interface import predict_folder
 from ....constants import LATENT_KEY
 from ....misc.toolkit import to_torch
 from ....misc.toolkit import to_device
 from ....misc.toolkit import eval_context
 
 
-class DINOPredictor:
+class DINOExtractor:
     def __init__(self, m: SimplePipeline, img_size: int, *, to_gray: bool = False):
         self.m = m
         self.dino = m.model
@@ -27,13 +28,13 @@ class DINOPredictor:
         return self.dino.device
 
     def get_latent(self, src_path: str) -> Tensor:
-        src = Image.open(src_path).convert("RGB")
+        src = to_rgb(Image.open(src_path))
         net = to_torch(self.transform(src)[None, ...]).to(self.device)
         with eval_context(self.dino):
             return self.dino.get_latent(net)
 
     def get_logits(self, src_path: str) -> Tensor:
-        src = Image.open(src_path).convert("RGB")
+        src = to_rgb(Image.open(src_path))
         net = self.transform(src)[None, ...].to(self.device)
         with eval_context(self.dino):
             return self.dino.get_logits(net)
@@ -83,5 +84,5 @@ class DINOPredictor:
 
 
 __all__ = [
-    "DINOPredictor",
+    "DINOExtractor",
 ]
