@@ -8,9 +8,11 @@ from typing import Union
 from typing import Callable
 from typing import Optional
 from cftool.misc import update_dict
+from cftool.misc import parse_config
 
 from ..models import *
 from ..types import data_type
+from ..types import configs_type
 from ..types import tensor_dict_type
 from ..types import general_config_type
 from ..types import sample_weights_type
@@ -26,7 +28,6 @@ from .zoo.core import DLZoo
 from ..data.interface import MLData
 from ..data.interface import CVDataModule
 from ..misc.toolkit import inject_debug
-from ..misc.toolkit import parse_config
 from ..misc.toolkit import download_model
 
 
@@ -102,8 +103,6 @@ def pack_onnx(
     opset: int = 11,
     simplify: bool = True,
     num_samples: Optional[int] = None,
-    compress: Optional[bool] = None,
-    remove_original: bool = True,
     verbose: bool = True,
     **kwargs: Any,
 ) -> DLPipeline:
@@ -121,11 +120,8 @@ def pack_onnx(
         onnx_file=onnx_file,
         opset=opset,
         simplify=simplify,
-        onnx_only=True,
         input_sample=input_sample,
         num_samples=num_samples,
-        compress=compress,
-        remove_original=remove_original,
         verbose=verbose,
         **kwargs,
     )
@@ -190,7 +186,8 @@ def fit_ml(
     clip_norm: float = 0.0,
     cudnn_benchmark: bool = False,
     metric_names: Optional[Union[str, List[str]]] = None,
-    metric_configs: Optional[Dict[str, Any]] = None,
+    metric_configs: configs_type = None,
+    metric_weights: Optional[Dict[str, float]] = None,
     use_losses_as_metrics: Optional[bool] = None,
     loss_metrics_weights: Optional[Dict[str, float]] = None,
     recompute_train_losses_in_eval: bool = True,
@@ -240,6 +237,7 @@ def fit_ml(
         cudnn_benchmark=cudnn_benchmark,
         metric_names=metric_names,
         metric_configs=metric_configs,
+        metric_weights=metric_weights,
         use_losses_as_metrics=use_losses_as_metrics,
         loss_metrics_weights=loss_metrics_weights,
         recompute_train_losses_in_eval=recompute_train_losses_in_eval,
@@ -328,7 +326,8 @@ def repeat_ml(
     clip_norm: float = 0.0,
     cudnn_benchmark: bool = False,
     metric_names: Optional[Union[str, List[str]]] = None,
-    metric_configs: Optional[Dict[str, Any]] = None,
+    metric_configs: configs_type = None,
+    metric_weights: Optional[Dict[str, float]] = None,
     use_losses_as_metrics: Optional[bool] = None,
     loss_metrics_weights: Optional[Dict[str, float]] = None,
     recompute_train_losses_in_eval: bool = True,
@@ -373,6 +372,7 @@ def repeat_ml(
         cudnn_benchmark=cudnn_benchmark,
         metric_names=metric_names,
         metric_configs=metric_configs,
+        metric_weights=metric_weights,
         use_losses_as_metrics=use_losses_as_metrics,
         loss_metrics_weights=loss_metrics_weights,
         recompute_train_losses_in_eval=recompute_train_losses_in_eval,
@@ -444,7 +444,8 @@ def fit_cv(
     clip_norm: float = 0.0,
     cudnn_benchmark: bool = False,
     metric_names: Optional[Union[str, List[str]]] = None,
-    metric_configs: Optional[Dict[str, Any]] = None,
+    metric_configs: configs_type = None,
+    metric_weights: Optional[Dict[str, float]] = None,
     use_losses_as_metrics: Optional[bool] = None,
     loss_metrics_weights: Optional[Dict[str, float]] = None,
     recompute_train_losses_in_eval: bool = True,
@@ -485,6 +486,7 @@ def fit_cv(
         cudnn_benchmark=cudnn_benchmark,
         metric_names=metric_names,
         metric_configs=metric_configs,
+        metric_weights=metric_weights,
         use_losses_as_metrics=use_losses_as_metrics,
         loss_metrics_weights=loss_metrics_weights,
         recompute_train_losses_in_eval=recompute_train_losses_in_eval,
@@ -773,6 +775,10 @@ def pixel_cnn(num_classes: int, **kwargs: Any) -> DLPipeline:
 # multimodal
 
 
+def clip(pretrained: bool = True, **kwargs: Any) -> DLPipeline:
+    return DLZoo.load_pipeline("multimodal/clip", pretrained=pretrained, **kwargs)
+
+
 def clip_model(pretrained: bool = True, **kwargs: Any) -> CLIP:
     return DLZoo.load_model("multimodal/clip", pretrained=pretrained, **kwargs)
 
@@ -810,7 +816,8 @@ def u2net_lite_model(pretrained: bool = False, **kwargs: Any) -> ModelProtocol:
 
 def u2net_finetune(ckpt: Optional[str] = None, **kwargs: Any) -> DLPipeline:
     if ckpt is None:
-        kwargs["pretrained_ckpt"] = download_model("u2net")
+        ckpt = download_model("u2net")
+    kwargs["pretrained_ckpt"] = ckpt
     return DLZoo.load_pipeline("segmentor/u2net.finetune", **kwargs)
 
 
@@ -951,6 +958,29 @@ def vq_vae_gray_lite(
     return _vq_vae("vq.gray_lite", img_size, num_classes, **kwargs)
 
 
+# nlp
+
+
+def hugging_face(model: str) -> DLPipeline:
+    return DLZoo.load_pipeline("hugging_face/general", model_config={"model": model})
+
+
+def hugging_face_model(model: str) -> HuggingFaceModel:
+    return DLZoo.load_model("hugging_face/general", model_config={"model": model})
+
+
+def simbert_model() -> SimBERT:
+    return DLZoo.load_model("hugging_face/simbert")
+
+
+def opus_model(src: str, tgt: str) -> OPUSBase:
+    return DLZoo.load_model("hugging_face/opus", model_config={"src": src, "tgt": tgt})
+
+
+def opus_zh_en_model() -> OPUS_ZH_EN:
+    return opus_model("zh", "en")
+
+
 __all__ = [
     "make",
     "run_ddp",
@@ -1006,4 +1036,9 @@ __all__ = [
     "vq_vae_gray",
     "vq_vae_lite",
     "vq_vae_gray_lite",
+    "hugging_face",
+    "hugging_face_model",
+    "simbert_model",
+    "opus_model",
+    "opus_zh_en_model",
 ]
