@@ -15,19 +15,21 @@ from typing import Optional
 from typing import NamedTuple
 from tqdm.autonotebook import tqdm
 from cfdata.tabular import TabularData
-from cftool.ml import ModelPattern
-from cftool.ml import EnsemblePattern
 from cftool.dist import Parallel
 from cftool.misc import update_dict
 from cftool.misc import shallow_copy_dict
-from cftool.ml.utils import patterns_type
-from cftool.ml.utils import Comparer
-from cftool.ml.utils import Estimator
+from cftool.misc import get_latest_workplace
+from cfml.misc.toolkit import patterns_type
+from cfml.misc.toolkit import Comparer
+from cfml.misc.toolkit import Estimator
+from cfml.misc.toolkit import ModelPattern
+from cfml.misc.toolkit import EnsemblePattern
 
 from .pipeline import SimplePipeline
 from .pipeline import CarefreePipeline
 from ...data import MLData
 from ...data import MLInferenceData
+from ...types import configs_type
 from ...trainer import get_sorted_checkpoints
 from ...constants import SCORES_FILE
 from ...constants import WARNING_PREFIX
@@ -36,7 +38,6 @@ from ...constants import ML_PIPELINE_SAVE_NAME
 from ...dist.ml import Experiment
 from ...dist.ml import ExperimentResults
 from ...misc.toolkit import to_2d
-from ...misc.toolkit import get_latest_workplace
 from ...models.ml.protocol import MLCoreProtocol
 
 
@@ -76,7 +77,7 @@ def evaluate(
     data: Union[MLData, MLInferenceData],
     *,
     metrics: Union[str, List[str]],
-    metric_configs: Optional[Union[Dict[str, Any], List[Dict[str, Any]]]] = None,
+    metric_configs: configs_type = None,
     contains_labels: bool = True,
     pipelines: Optional[various_pipelines_type] = None,
     predict_config: Optional[Dict[str, Any]] = None,
