@@ -13,14 +13,19 @@ from typing import Optional
 from functools import partial
 from collections import OrderedDict
 from cfdata.tabular import ColumnTypes
-from cftool.ml import ModelPattern
+from cftool.misc import get_arguments
 from cftool.misc import lock_manager
 from cftool.misc import Saving
+from cftool.array import softmax
+from cftool.array import squeeze
+from cftool.array import is_float
+from cfml.misc.toolkit import ModelPattern
 
 from ...data import MLData
 from ...data import MLLoader
 from ...data import MLDataset
 from ...data import DLDataModule
+from ...types import configs_type
 from ...types import np_dict_type
 from ...types import states_callback_type
 from ...trainer import get_sorted_checkpoints
@@ -29,10 +34,6 @@ from ...protocol import InferenceOutputs
 from ...constants import PT_PREFIX
 from ...constants import SCORES_FILE
 from ...constants import PREDICTIONS_KEY
-from ...misc.toolkit import softmax
-from ...misc.toolkit import squeeze
-from ...misc.toolkit import is_float
-from ...misc.toolkit import get_arguments
 from ...misc.internal_ import MLInference
 from ...models.ml.encoders import Encoder
 from ...models.ml.protocol import MLModel
@@ -78,7 +79,8 @@ class SimplePipeline(DLPipeline):
         clip_norm: float = 0.0,
         cudnn_benchmark: bool = False,
         metric_names: Optional[Union[str, List[str]]] = None,
-        metric_configs: Optional[Dict[str, Any]] = None,
+        metric_configs: configs_type = None,
+        metric_weights: Optional[Dict[str, float]] = None,
         use_losses_as_metrics: Optional[bool] = None,
         loss_metrics_weights: Optional[Dict[str, float]] = None,
         recompute_train_losses_in_eval: bool = True,
@@ -117,6 +119,7 @@ class SimplePipeline(DLPipeline):
             cudnn_benchmark=cudnn_benchmark,
             metric_names=metric_names,
             metric_configs=metric_configs,
+            metric_weights=metric_weights,
             use_losses_as_metrics=use_losses_as_metrics,
             loss_metrics_weights=loss_metrics_weights,
             recompute_train_losses_in_eval=recompute_train_losses_in_eval,
@@ -336,7 +339,8 @@ class CarefreePipeline(SimplePipeline):
         clip_norm: float = 0.0,
         cudnn_benchmark: bool = False,
         metric_names: Optional[Union[str, List[str]]] = None,
-        metric_configs: Optional[Dict[str, Any]] = None,
+        metric_configs: configs_type = None,
+        metric_weights: Optional[Dict[str, float]] = None,
         use_losses_as_metrics: Optional[bool] = None,
         loss_metrics_weights: Optional[Dict[str, float]] = None,
         recompute_train_losses_in_eval: bool = True,
@@ -385,6 +389,7 @@ class CarefreePipeline(SimplePipeline):
             cudnn_benchmark=cudnn_benchmark,
             metric_names=metric_names,
             metric_configs=metric_configs,
+            metric_weights=metric_weights,
             use_losses_as_metrics=use_losses_as_metrics,
             loss_metrics_weights=loss_metrics_weights,
             recompute_train_losses_in_eval=recompute_train_losses_in_eval,
