@@ -23,6 +23,7 @@ from typing import Callable
 from typing import Optional
 from typing import NamedTuple
 from cftool.dist import Parallel
+from cftool.misc import walk
 from cftool.misc import shallow_copy_dict
 from cftool.misc import Saving
 from torch.utils.data import Dataset
@@ -50,7 +51,6 @@ from ..constants import INFO_PREFIX
 from ..constants import ERROR_PREFIX
 from ..constants import WARNING_PREFIX
 from ..constants import ORIGINAL_LABEL_KEY
-from ..misc.toolkit import walk
 
 
 # dl
@@ -68,12 +68,10 @@ class TensorDataset(Dataset):
         self.others = others
 
     def __getitem__(self, index: int) -> tensor_dict_type:
-        label = 0 if self.y is None else self.y[index]
-        item = {
-            INPUT_KEY: self.x[index],
-            LABEL_KEY: label,
-            ORIGINAL_LABEL_KEY: label,
-        }
+        item = {INPUT_KEY: self.x[index]}
+        if self.y is not None:
+            label = self.y[index]
+            item.update({LABEL_KEY: label, ORIGINAL_LABEL_KEY: label})
         if self.others is not None:
             for k, v in self.others.items():
                 item[k] = v[index]
@@ -83,6 +81,27 @@ class TensorDataset(Dataset):
         return self.x.shape[0]
 
 
+class TensorDictDataset(Dataset):
+    def __init__(
+        self,
+        x: tensor_dict_type,
+        y: Optional[Tensor],
+    ):
+        self.x = x
+        self.y = y
+        self.x_keys = sorted(self.x)
+
+    def __getitem__(self, index: int) -> tensor_dict_type:
+        item = {k: self.x[k][index] for k in self.x_keys}
+        if self.y is not None:
+            label = self.y[index]
+            item.update({LABEL_KEY: label, ORIGINAL_LABEL_KEY: label})
+        return item
+
+    def __len__(self) -> int:
+        return self.x[self.x_keys[0]].shape[0]
+
+
 @DLDataModule.register("tensor")
 class TensorData(DLDataModule):
     def __init__(
@@ -131,6 +150,50 @@ class TensorData(DLDataModule):
         return train_loader, valid_loader
 
 
+@DLDataModule.register("tensor_dict")
+class TensorDictData(DLDataModule):
+    def __init__(
+        self,
+        x_train: tensor_dict_type,
+        y_train: Optional[Tensor] = None,
+        x_valid: Optional[tensor_dict_type] = None,
+        y_valid: Optional[Tensor] = None,
+        *,
+        shuffle: bool = True,
+        batch_size: int = 64,
+        num_workers: int = 0,
+    ):
+        self.x_train = x_train
+        self.y_train = y_train
+        self.x_valid = x_valid
+        self.y_valid = y_valid
+        self.kw = dict(batch_size=batch_size, shuffle=shuffle, num_workers=num_workers)
+        self.test_transform = None
+
+    @property
+    def info(self) -> Dict[str, Any]:
+        return self.kw
+
+    # TODO : support sample weights
+    def prepare(self, sample_weights: sample_weights_type) -> None:
+        def _get_data(x: Any, y: Any) -> DLDataset:
+            return DLDataset(TensorDictDataset(x, y))
+
+        self.train_data = _get_data(self.x_train, self.y_train)
+        if self.x_valid is None:
+            self.valid_data = None
+        else:
+            self.valid_data = _get_data(self.x_valid, self.y_valid)
+
+    def initialize(self) -> Tuple[DLLoader, Optional[DLLoader]]:
+        train_loader = DLLoader(DataLoader(self.train_data, **self.kw))  # type: ignore
+        if self.valid_data is None:
+            valid_loader = None
+        else:
+            valid_loader = DLLoader(DataLoader(self.valid_data, **self.kw))  # type: ignore
+        return train_loader, valid_loader
+
+
 @DLDataModule.register("dummy")
 class DummyData(TensorData):
     def __init__(
@@ -945,6 +1008,7 @@ def prepare_image_folder_data(
 
 __all__ = [
     "TensorData",
+    "TensorDictData",
     "DummyData",
     "MLData",
     "MLInferenceData",
