@@ -16,16 +16,16 @@ from typing import Tuple
 from typing import Union
 from typing import Optional
 from typing import NamedTuple
+from cfc.stat import RollingStat
 from cftool.misc import hash_code
 from cftool.misc import fix_float_to_length
-from cftool.misc import StrideArray
 from cftool.misc import LoggingMixin
-from cftool.stat import RollingStat
+from cftool.misc import WithRegister
+from cftool.array import squeeze
+from cftool.array import StrideArray
+from cftool.array import SharedArrayWrapper
 
-from ..toolkit import squeeze
 from ..toolkit import to_torch
-from ..toolkit import WithRegister
-from ..toolkit import SharedArrayWrapper
 from ...data import get_weighted_indices
 from ...data import MLDataset
 from ...types import tensor_dict_type
@@ -333,7 +333,7 @@ class TimeSeriesDataManager(LoggingMixin):
             valid_mask = x_mask & y_mask
         else:
             offset = x_window + self.cfg.y_span - 1
-            offset_valid = np.zeros_like(y_mask, np.bool_)
+            offset_valid = np.zeros_like(y_mask, bool)
             offset_valid[x_window - 1 : -self.cfg.y_span] = y_mask[offset:]
             valid_mask = x_mask & offset_valid
         # sanity check
@@ -425,7 +425,7 @@ class TimeSeriesDataManager(LoggingMixin):
             padding = self.cfg.xy_padding + 1
             offset = offset_start + padding
             for mask in masks:
-                local_valid = np.zeros_like(mask, np.bool_)
+                local_valid = np.zeros_like(mask, bool)
                 local_valid[offset_start:-padding] = mask[offset:]
                 valid_mask &= local_valid
         valid_mask[begin : begin + self.cfg.x_window - 1] = False
