@@ -8,12 +8,12 @@ from typing import Dict
 from typing import Type
 from typing import Optional
 from cftool.misc import shallow_copy_dict
+from cftool.misc import WithRegister
 
 from ....types import tensor_dict_type
 from ....protocol import TrainerState
 from ....misc.toolkit import interpolate
 from ....misc.toolkit import auto_num_layers
-from ....misc.toolkit import WithRegister
 from ....constants import INPUT_KEY
 from ....constants import LABEL_KEY
 from ....constants import PREDICTIONS_KEY
