@@ -8,8 +8,8 @@ from typing import List
 from typing import Type
 from typing import Optional
 from typing import NamedTuple
+from cftool.misc import WithRegister
 
-from ....misc.toolkit import WithRegister
 from ....modules.blocks import get_conv_blocks
 from ....modules.blocks import Conv2d
 
