@@ -12,6 +12,7 @@ from typing import Tuple
 from typing import Optional
 from cftool.misc import update_dict
 from cftool.misc import shallow_copy_dict
+from cftool.array import l2_normalize
 from torch.nn.parallel import DistributedDataParallel as DDP
 
 from ..encoder import Encoder1DBase
@@ -25,7 +26,6 @@ from ....constants import LOSS_KEY
 from ....constants import INPUT_KEY
 from ....constants import LATENT_KEY
 from ....misc.toolkit import to_device
-from ....misc.toolkit import l2_normalize
 from ....misc.toolkit import get_world_size
 from ....misc.toolkit import has_batch_norms
 
