@@ -1,4 +1,6 @@
+import os
 import math
+import onnx
 import torch
 
 import numpy as np
@@ -17,8 +19,12 @@ from typing import TypeVar
 from typing import Callable
 from typing import Optional
 from typing import NamedTuple
+from onnxsim import simplify as onnx_simplify
+from cftool.misc import lock_manager
 from cftool.misc import shallow_copy_dict
 from cftool.misc import context_error_handler
+from cftool.misc import WithRegister
+from onnxsim.onnx_simplifier import get_input_names
 
 from .types import losses_type
 from .types import np_dict_type
@@ -26,14 +32,16 @@ from .types import tensor_dict_type
 from .constants import LOSS_KEY
 from .constants import INPUT_KEY
 from .constants import LABEL_KEY
+from .constants import INFO_PREFIX
+from .constants import WARNING_PREFIX
 from .constants import PREDICTIONS_KEY
 from .constants import BATCH_INDICES_KEY
 from .misc.toolkit import to_numpy
 from .misc.toolkit import to_device
 from .misc.toolkit import eval_context
 from .misc.toolkit import get_world_size
+from .misc.toolkit import fix_denormal_states
 from .misc.toolkit import ONNX
-from .misc.toolkit import WithRegister
 
 
 dataset_dict: Dict[str, Type["DatasetProtocol"]] = {}
@@ -134,6 +142,115 @@ class ModelProtocol(nn.Module, WithRegister["ModelProtocol"], metaclass=ABCMeta)
     def summary_forward(self, batch_idx: int, batch: tensor_dict_type) -> None:
         self.forward(batch_idx, batch)
 
+    def to_onnx(
+        self,
+        export_folder: str,
+        input_sample: tensor_dict_type,
+        dynamic_axes: Optional[Union[List[int], Dict[int, str]]] = None,
+        *,
+        onnx_file: str = "model.onnx",
+        opset: int = 11,
+        simplify: bool = True,
+        forward_fn: Optional[Callable[[Dict[str, Any]], Dict[str, Any]]] = None,
+        output_names: Optional[List[str]] = None,
+        num_samples: Optional[int] = None,
+        verbose: bool = True,
+        **kwargs: Any,
+    ) -> "ModelProtocol":
+        # prepare
+        device = self.device
+        model = self.cpu()
+        if num_samples is not None:
+            input_sample = {k: v[:num_samples] for k, v in input_sample.items()}
+        onnx_forward = forward_fn or model.onnx_forward
+        input_names = sorted(input_sample.keys())
+        if output_names is None:
+            if forward_fn is not None:
+                msg = "`output_names` should be provided when `forward_fn` is provided"
+                raise ValueError(msg)
+            with eval_context(model):
+                forward_results = onnx_forward(shallow_copy_dict(input_sample))
+            if not isinstance(forward_results, dict):
+                forward_results = {PREDICTIONS_KEY: forward_results}
+            output_names = sorted(forward_results.keys())
+        # setup
+        kwargs = shallow_copy_dict(kwargs)
+        kwargs["input_names"] = input_names
+        kwargs["output_names"] = output_names
+        kwargs["opset_version"] = opset
+        kwargs["export_params"] = True
+        kwargs["do_constant_folding"] = True
+        if dynamic_axes is None:
+            dynamic_axes = {}
+        elif isinstance(dynamic_axes, list):
+            dynamic_axes = {axis: f"axis.{axis}" for axis in dynamic_axes}
+        if num_samples is None:
+            dynamic_axes[0] = "batch_size"
+        dynamic_axes_settings = {}
+        for name in input_names + output_names:
+            dynamic_axes_settings[name] = dynamic_axes
+        kwargs["dynamic_axes"] = dynamic_axes_settings
+        kwargs["verbose"] = verbose
+        # export
+        abs_folder = os.path.abspath(export_folder)
+        base_folder = os.path.dirname(abs_folder)
+
+        class ONNXWrapper(torch.nn.Module):
+            def __init__(self) -> None:
+                super().__init__()
+                self.model = model
+
+            def forward(self, batch: Dict[str, Any]) -> Dict[str, Any]:
+                rs = onnx_forward(batch)
+                if isinstance(rs, torch.Tensor):
+                    return {k: rs for k in output_names}  # type: ignore
+                return {k: rs[k] for k in output_names}  # type: ignore
+
+        with lock_manager(base_folder, []) as lock:
+            onnx_path = os.path.join(export_folder, onnx_file)
+            lock._stuffs = [onnx_path]
+            os.makedirs(export_folder, exist_ok=True)
+            m_onnx = ONNXWrapper()
+            original_states = model.state_dict()
+            fixed_states = fix_denormal_states(original_states, verbose=verbose)
+            with eval_context(m_onnx):
+                model.load_state_dict(fixed_states)
+                torch.onnx.export(
+                    m_onnx,
+                    ({k: input_sample[k] for k in input_names}, {}),
+                    onnx_path,
+                    **shallow_copy_dict(kwargs),
+                )
+                model.load_state_dict(original_states)
+                onnx_model = onnx.load(onnx_path)
+                input_names = get_input_names(onnx_model)
+                np_sample = {
+                    name: to_numpy(tensor)
+                    for name, tensor in input_sample.items()
+                    if name in input_names
+                }
+                try:
+                    if not simplify:
+                        model_simplified = onnx_model
+                        check = True
+                    else:
+                        model_simplified, check = onnx_simplify(
+                            onnx_model,
+                            input_data=np_sample,
+                            dynamic_input_shape=bool(dynamic_axes),
+                        )
+                except Exception as err:
+                    if verbose:
+                        print(f"{WARNING_PREFIX}Failed to simplify ONNX model ({err})")
+                    check = False
+                if not check and verbose:
+                    print(f"{INFO_PREFIX}Simplified ONNX model is not validated!")
+                elif check and verbose and simplify:
+                    print(f"{INFO_PREFIX}Simplified ONNX model is validated!")
+                    onnx_model = model_simplified
+                onnx.save(onnx_model, onnx_path)
+        return self.to(device)
+
 
 class StepOutputs(NamedTuple):
     forward_results: tensor_dict_type
@@ -466,7 +583,7 @@ class MultiLoss(LossProtocol, metaclass=ABCMeta):
         if tag in cls.d:
             return tag
 
-        @cls.register(tag)
+        @cls.register(tag)  # type: ignore
         class _(cls):  # type: ignore
             names = base_loss_names
 
@@ -536,7 +653,7 @@ class AuxLoss(LossProtocol):
         if tag in cls.d:
             return tag
 
-        @cls.register(tag)
+        @cls.register(tag)  # type: ignore
         class _(cls):  # type: ignore
             loss_name = base_loss_name
             aux_names = aux_loss_names
@@ -584,6 +701,7 @@ class InferenceProtocol:
         metrics: Optional["MetricProtocol"] = None,
         loss: Optional[LossProtocol] = None,
         return_outputs: bool = True,
+        stack_outputs: bool = True,
         use_tqdm: bool = False,
         **kwargs: Any,
     ) -> InferenceOutputs:
@@ -672,7 +790,9 @@ class InferenceProtocol:
                 final_results = {k: None for k in results}
             else:
                 final_results = {
-                    batch_key: np.vstack(batch_results)
+                    batch_key: batch_results
+                    if not stack_outputs
+                    else np.vstack(batch_results)
                     if isinstance(batch_results[0], np.ndarray)
                     else [
                         np.vstack([batch[i] for batch in batch_results])
