@@ -13,6 +13,7 @@ from typing import Optional
 
 arr_type = Union[np.ndarray, torch.Tensor]
 data_type = Optional[Union[np.ndarray, str]]
+texts_type = Union[str, List[str]]
 param_type = Union[torch.Tensor, torch.nn.Parameter]
 configs_type = Optional[Union[List[Dict[str, Any]], Dict[str, Any]]]
 general_config_type = Optional[Union[str, Dict[str, Any]]]
@@ -26,6 +27,7 @@ sample_weights_type = Optional[Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]]
 __all__ = [
     "arr_type",
     "data_type",
+    "texts_type",
     "param_type",
     "configs_type",
     "general_config_type",
